#include "Gadget/Engine.h"
#include "Particle.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G;
	G.Window.SetSize(1024, 768);
	G.Window.Open();
	G.Mouse.SetVisibility(true); //I want to see the mouse pointer
	glDisable(GL_DEPTH_TEST); //Ignore depth

	//Init is a static function so I can call it before I have created any particles
	Particle::Init(&G); //Calling this function causes the image to load

	list<Particle> particles; //Create an empty list of Particles

	while(G.Window.IsOpen())//loop as long as the window is open
	{
		G.Renderer.BeginFrame(); G.Renderer.Render2D();

			//ESC closes the window
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	

			//Create new particles
			if(G.Mouse.GetLeftHold())
			{
				Particle tempPart;
				//Set the position and velocity
				tempPart.position = G.Mouse.GetPosition();
				tempPart.velocity = tempPart.velocity.AngleToVector(float(rand()%360))*100.0f;
				tempPart.velocity.y += 300.0f;
				particles.push_back(tempPart); //Copy this particle into the list
			}

			//Create a bunch of particles that form a circle
			if(G.Mouse.GetRightPress())
			{
				for(int i = 0; i<36; i++)//Create 36 particles at once!
				{
					Particle tempPart;
					//Set the position and velocity
					tempPart.position = G.Mouse.GetPosition();
					tempPart.velocity = tempPart.velocity.AngleToVector(float(i*10))*200.0f;
					particles.push_back(tempPart); //Copy this particle into the list
				}
			}

			//Loop through the list of particles and call their step function
			for(list<Particle>::iterator i = particles.begin(); i != particles.end();)
			{
				if(i->timer>7.0f)//Erase the particle if it has been alive for more than 3 seconds
				{
					i = particles.erase(i); //The erase function returns the next iterator in the list
				}
				else
				{
					i->Step(); //Notice that the iterator can be used like a pointer
					i++; //Move on to the next particle in the list
				}
			}

			Shader::Deactivate();//Always a good idea to turn off any shaders when finished rendering
			
		G.Renderer.EndFrame();	
	}
	return 0;
}			